require 'spec_helper'

describe SuckerPunch::Persistence do
  class TestJob
    include SuckerPunch::Job
    prepend ::PersistenceSupport::TestDataStore

    def perform(*args)
      'Hamato Yoshi'
    end
  end

  it 'persists asynchronous jobs as they are queued' do
    PersistenceSupport::TestDataStore
      .should_receive(:create_record)
      .with(['Donatello'])

    TestJob.new.async.perform('Donatello')
  end

  it 'persists synchronous jobs when they are queued' do
    PersistenceSupport::TestDataStore
      .should_receive(:create_record)
      .with(['Raphael'])

    TestJob.new.perform('Raphael')
  end

  it 'marks job records as started before executing them' do
    PersistenceSupport::TestDataStore
      .should_receive(:update_status)
      .with({ data: ['Raphael']}, 'started')
      .ordered

    PersistenceSupport::TestDataStore
      .should_receive(:update_status)
      .with(any_args).ordered

    TestJob.new.perform('Raphael')
  end

  it 'marks job records as finished after executing them' do
    PersistenceSupport::TestDataStore
      .should_receive(:update_status)
      .with(any_args).once.ordered

    PersistenceSupport::TestDataStore
      .should_receive(:update_status)
      .with({ data: ['Leonardo']}, 'complete', 'Hamato Yoshi')
      .ordered

    TestJob.new.perform('Leonardo')
  end
end
